@@ -8,24 +8,22 @@ use strict;
 use Mouse;
 use JSON qw(from_json to_json);
 use Lemonldap::NG::Portal::Main::Constants qw(
+  PE_OK
   PE_ERROR
   PE_BADOTP
   PE_FORMEMPTY
-  PE_OK
   PE_SENDRESPONSE
 );
 
-our $VERSION = '2.0.9';
+our $VERSION = '2.0.12';
 
 extends 'Lemonldap::NG::Portal::Main::SecondFactor';
 
 # INITIALIZATION
 
 has prefix => ( is => 'ro', default => 'yubikey' );
-
-has logo => ( is => 'rw', default => 'yubikey.png' );
-
-has yubi => ( is => 'rw' );
+has logo   => ( is => 'rw', default => 'yubikey.png' );
+has yubi   => ( is => 'rw' );
 
 sub init {
     my ($self) = @_;
@@ -79,7 +77,7 @@ sub init {
                     $self->conf->{yubikey2fUrl}
                     ? ( url => $self->conf->{yubikey2fUrl} )
                     : ()
-                ),
+                )
             }
         )
     );
@@ -88,8 +86,7 @@ sub init {
 
 sub _findYubikey {
     my ( $self, $req, $sessionInfo ) = @_;
-    my $yubikey;
-    my $_2fDevices;
+    my ( $yubikey, $_2fDevices );
 
     # First, lookup from session attribute
     if ( $self->conf->{yubikey2fFromSessionAttribute} ) {
@@ -99,7 +96,7 @@ sub _findYubikey {
 
     # If we didn't find a key, lookup psession
     if ( !$yubikey and $sessionInfo->{_2fDevices} ) {
-        $self->logger->debug("Loading 2F Devices ...");
+        $self->logger->debug("Loading 2F Devices...");
 
         # Read existing 2FDevices
         $_2fDevices = eval {
@@ -110,10 +107,19 @@ sub _findYubikey {
             return PE_ERROR;
         }
         $self->logger->debug("2F Device(s) found");
-        $self->logger->debug("Reading Yubikey ...");
+        $self->logger->debug("Reading Yubikey...");
 
-        $yubikey = $_->{_yubikey}
-          foreach grep { $_->{type} eq 'UBK' } @$_2fDevices;
+        if ( my $code = $req->param('code') ) {
+            $yubikey = $_->{_yubikey} foreach grep {
+                ( $_->{type} eq 'UBK' )
+                  and ( $_->{_yubikey} eq
+                    substr( $code, 0, $self->conf->{yubikey2fPublicIDSize} ) )
+            } @$_2fDevices;
+        }
+        else {
+            $yubikey = $_->{_yubikey}
+              foreach grep { $_->{type} eq 'UBK' } @$_2fDevices;
+        }
     }
 
     return $yubikey;
@@ -124,7 +130,10 @@ sub run {
     my ( $self, $req, $token, $_2fDevices ) = @_;
 
     my $checkLogins = $req->param('checkLogins');
-    $self->logger->debug("Yubikey checkLogins set") if ($checkLogins);
+    $self->logger->debug("Yubikey; checkLogins set") if $checkLogins;
+
+    my $stayconnected = $req->param('stayconnected');
+    $self->logger->debug("Yubikey: stayconnected set") if $stayconnected;
 
     my $yubikey = $self->_findYubikey( $req, $req->sessionInfo );
 
@@ -141,13 +150,14 @@ sub run {
         $req,
         'ext2fcheck',
         params => {
-            MAIN_LOGO   => $self->conf->{portalMainLogo},
-            SKIN        => $self->p->getSkin($req),
-            TOKEN       => $token,
-            TARGET      => '/yubikey2fcheck?skin=' . $self->p->getSkin($req),
-            INPUTLOGO   => 'yubikey.png',
-            LEGEND      => 'clickOnYubikey',
-            CHECKLOGINS => $checkLogins
+            MAIN_LOGO     => $self->conf->{portalMainLogo},
+            SKIN          => $self->p->getSkin($req),
+            TOKEN         => $token,
+            TARGET        => '/yubikey2fcheck?skin=' . $self->p->getSkin($req),
+            INPUTLOGO     => 'yubikey.png',
+            LEGEND        => 'clickOnYubikey',
+            CHECKLOGINS   => $checkLogins,
+            STAYCONNECTED => $stayconnected
         }
     );
     $self->logger->debug("Display Yubikey form");
