@@ -2,7 +2,7 @@
 # into "plugins" list in lemonldap-ng.ini, section "portal"
 package Lemonldap::NG::Portal::Main::Plugins;
 
-our $VERSION = '2.0.10';
+our $VERSION = '2.0.12';
 
 package Lemonldap::NG::Portal::Main;
 
@@ -28,13 +28,16 @@ our @pList = (
     checkState                          => '::Plugins::CheckState',
     portalForceAuthn                    => '::Plugins::ForceAuthn',
     checkUser                           => '::Plugins::CheckUser',
+    checkDevOps                         => '::Plugins::CheckDevOps',
     impersonationRule                   => '::Plugins::Impersonation',
     contextSwitchingRule                => '::Plugins::ContextSwitching',
     decryptValueRule                    => '::Plugins::DecryptValue',
+    findUser                            => '::Plugins::FindUser',
     adaptativeAuthenticationLevelRules =>
       '::Plugins::AdaptativeAuthenticationLevel',
     globalLogoutRule => '::Plugins::GlobalLogout',
     refreshSessions  => '::Plugins::Refresh',
+    crowdsec         => '::Plugins::CrowdSec',
 );
 
 ##@method list enabledPlugins
@@ -62,7 +65,15 @@ sub enabledPlugins {
 
     # Load static plugin list
     for ( my $i = 0 ; $i < @pList ; $i += 2 ) {
-        push @res, $pList[ $i + 1 ] if ( $conf->{ $pList[$i] } );
+        my $pluginConf = $conf->{ $pList[$i] };
+        if ( ref($pluginConf) eq "HASH" ) {
+
+            # Do not load plugin if config is an empty hash
+            push @res, $pList[ $i + 1 ] if %{$pluginConf};
+        }
+        else {
+            push @res, $pList[ $i + 1 ] if $pluginConf;
+        }
     }
 
     # Check if SOAP is enabled
@@ -73,8 +84,9 @@ sub enabledPlugins {
     # Add REST (check is done by it)
     push @res, '::Plugins::RESTServer';
 
+    # Check if password is enabled
     if ( my $p = $conf->{passwordDB} ) {
-        push @res, "::Password::$p" if ( $p ne 'Null' );
+        push @res, "::Password::$p";
     }
 
     # Check if register is enabled
@@ -82,11 +94,11 @@ sub enabledPlugins {
       if ( $conf->{registerDB} and $conf->{registerDB} ne 'Null' );
 
     # Check if custom plugins are required
-    # TODO: change this name
     if ( $conf->{customPlugins} ) {
         $self->logger->debug( 'Custom plugins: ' . $conf->{customPlugins} );
         push @res, grep ( /\w+/, split( /,\s*/, $conf->{customPlugins} ) );
     }
+    
     return @res;
 }
 
