@@ -5,8 +5,8 @@ use strict;
 use Mouse;
 use Lemonldap::NG::Portal::Main::Constants qw(
   PE_OK
-  PE_BADOLDPASSWORD
   PE_PASSWORD_OK
+  PE_BADOLDPASSWORD
   PE_PASSWORD_MISMATCH
   PE_PP_PASSWORD_TOO_SHORT
   PE_PP_NOT_ALLOWED_CHARACTER
@@ -17,12 +17,23 @@ use Lemonldap::NG::Portal::Main::Constants qw(
 
 extends 'Lemonldap::NG::Portal::Main::Plugin';
 
-our $VERSION = '2.0.8';
+our $VERSION = '2.0.12';
 
 # INITIALIZATION
 
+has requireOldPwdRule => ( is => 'rw' );
+
 sub init {
-    $_[0]->p->{_passwordDB} = $_[0];
+    my ($self) = shift;
+    $self->requireOldPwdRule(
+        $self->p->buildRule(
+            $self->conf->{portalRequireOldPassword},
+            'portalRequireOldPassword'
+        )
+    );
+    return 0 unless $self->requireOldPwdRule;
+
+    $self->p->{_passwordDB} = $self;
 }
 
 # INTERFACE
@@ -42,17 +53,26 @@ sub _modifyPassword {
     return PE_PASSWORD_MISMATCH
       unless ( $req->data->{newpassword} eq $req->param('confirmpassword') );
 
-    my $rule = $self->p->HANDLER->buildSub(
+    my $oldPwdRule = $self->p->HANDLER->buildSub(
         $self->p->HANDLER->substitute(
             $self->conf->{portalRequireOldPassword}
         )
     );
-    unless ($rule) {
+    unless ($oldPwdRule) {
+        my $error = $self->p->HANDLER->tsv->{jail}->error || '???';
+    }
+
+    my $pwdPolicyRule = $self->p->HANDLER->buildSub(
+        $self->p->HANDLER->substitute(
+            $self->conf->{passwordPolicyActivation}
+        )
+    );
+    unless ($pwdPolicyRule) {
         my $error = $self->p->HANDLER->tsv->{jail}->error || '???';
     }
 
     # Check if portal require old password
-    if ( $rule->( $req, $req->userData ) or $requireOldPwd ) {
+    if ( $oldPwdRule->( $req, $req->userData ) or $requireOldPwd ) {
 
         # TODO: verify oldpassword
         unless ( $req->data->{oldpassword} = $req->param('oldpassword') ) {
@@ -65,13 +85,34 @@ sub _modifyPassword {
           unless ( $self->confirm( $req, $req->data->{oldpassword} ) );
     }
 
-    my $cpq = $self->checkPasswordQuality( $req->data->{newpassword} );
+    my $cpq =
+        $pwdPolicyRule->( $req, $req->userData )
+      ? $self->checkPasswordQuality( $req->data->{newpassword} )
+      : PE_OK;
     return $cpq unless ( $cpq == PE_OK );
 
+    my $hook_result = $self->p->processHook(
+        $req, 'passwordBeforeChange', $req->user,
+        $req->data->{newpassword},
+        $req->data->{oldpassword}
+    );
+    return $hook_result if ( $hook_result != PE_OK );
+
     # Call password package
     my $res = $self->modifyPassword( $req, $req->data->{newpassword} );
     if ( $res == PE_PASSWORD_OK ) {
+
+        $self->p->processHook(
+            $req, 'passwordAfterChange', $req->user,
+            $req->data->{newpassword},
+            $req->data->{oldpassword}
+        );
+
         $self->logger->debug( 'Update password in session for ' . $req->user );
+        my $userlog = $req->sessionInfo->{ $self->conf->{whatToTrace} };
+        my $iplog   = $req->sessionInfo->{ipAddr};
+        $self->userLogger->notice("Password changed for $userlog ($iplog)")
+          if ( defined $userlog and $iplog );
         my $infos;
 
         # Store new password if asked
@@ -80,7 +121,7 @@ sub _modifyPassword {
                 $req,
                 {
                     _passwordDB => $self->p->getModule( $req, 'password' ),
-                    _password   => $req->{newpassword}
+                    _password   => $req->data->{newpassword}
                 }
             );
         }
@@ -142,34 +183,74 @@ sub checkPasswordQuality {
         }
     }
 
-    ## Special characters policy
+    ### Special characters policy
     my $speChars = $self->conf->{passwordPolicySpecialChar};
     $speChars =~ s/\s+//g;
 
-    # Min special characters
+    ## Min special characters
+    # Just number of special characters must be checked
+    if ( $self->conf->{passwordPolicyMinSpeChar} && $speChars eq '__ALL__' ) {
+        my $spe = $password =~ s/\W//g;
+        if ( $spe < $self->conf->{passwordPolicyMinSpeChar} ) {
+            $self->logger->error("Password has not enough special characters");
+            return PE_PP_INSUFFICIENT_PASSWORD_QUALITY;
+        }
+        return PE_OK;
+    }
+
+    # Number of special characters must be checked
     if ( $self->conf->{passwordPolicyMinSpeChar} && $speChars ) {
-        my $spe  = 0;
         my $test = $password;
-        $spe = $test =~ s/[\Q$speChars\E]//g;
+        my $spe  = $test =~ s/[\Q$speChars\E]//g;
         if ( $spe < $self->conf->{passwordPolicyMinSpeChar} ) {
             $self->logger->error("Password has not enough special characters");
             return PE_PP_INSUFFICIENT_PASSWORD_QUALITY;
         }
     }
 
-    # Fobidden special characters
-    $password =~ s/[\Q$speChars\E\w]//g;
-    if ($password) {
-        $self->logger->error( 'Password contains '
-              . length($password)
-              . " forbidden character(s): $password" );
-        return
-          length($password) > 1
-          ? PE_PP_NOT_ALLOWED_CHARACTERS
-          : PE_PP_NOT_ALLOWED_CHARACTER;
+    ## Fobidden special characters
+    unless ( $speChars eq '__ALL__' ) {
+        $password =~ s/[\Q$speChars\E\w]//g;
+        if ($password) {
+            $self->logger->error( 'Password contains '
+                  . length($password)
+                  . " forbidden character(s): $password" );
+            return length($password) > 1
+              ? PE_PP_NOT_ALLOWED_CHARACTERS
+              : PE_PP_NOT_ALLOWED_CHARACTER;
+        }
     }
 
     return PE_OK;
 }
 
+# This method should be called when resetting the password
+# in order to call the password hook
+sub setNewPassword {
+    my ( $self, $req, $pwd, $useMail ) = @_;
+
+    my $hook_result =
+      $self->p->processHook( $req, 'passwordBeforeChange', $req->user, $pwd );
+    return $hook_result if ( $hook_result != PE_OK );
+
+    # Delegate to subclass
+    my $mod_result = $self->modifyPassword( $req, $pwd, $useMail );
+
+    if ( $mod_result == PE_PASSWORD_OK ) {
+        $hook_result =
+          $self->p->processHook( $req, 'passwordAfterChange', $req->user,
+            $pwd );
+        if ( $hook_result != PE_OK ) {
+            return $hook_result;
+        }
+        else {
+            return PE_PASSWORD_OK;
+        }
+    }
+    else {
+        return $mod_result;
+    }
+
+}
+
 1;
