@@ -4,10 +4,12 @@ use strict;
 use Mouse;
 use Lemonldap::NG::Portal::Main::Constants qw(
   PE_APACHESESSIONERROR
+  PE_ERROR
   PE_OK
+  URIRE
 );
 
-our $VERSION = '2.0.8';
+our $VERSION = '2.0.12';
 
 extends 'Lemonldap::NG::Common::Module';
 
@@ -24,7 +26,8 @@ sub changeUrldc {
     my ( $self, $req ) = @_;
     my $urldc = $req->{urldc} || '';
     if (    $req->id
-        and $urldc !~ m#^https?://[^/]*$self->{conf}->{domain}(:\d+)?/#oi
+        and $urldc =~ URIRE
+        and $3 !~ m@\Q$self->{conf}->{domain}\E$@oi
         and $self->p->isTrustedUrl($urldc) )
     {
         my $ssl = $urldc =~ /^https/;
@@ -37,9 +40,18 @@ sub changeUrldc {
             $cdaInfos->{cookie_name}  = $self->{conf}->{cookieName};
         }
         else {
-            $cdaInfos->{cookie_value} =
-              $req->{sessionInfo}->{_httpSession};
-            $cdaInfos->{cookie_name} = $self->{conf}->{cookieName} . "http";
+            if ( $req->{sessionInfo}->{_httpSession} ) {
+                $cdaInfos->{cookie_value} =
+                  $req->{sessionInfo}->{_httpSession};
+                $cdaInfos->{cookie_name} = $self->{conf}->{cookieName} . "http";
+            }
+            else {
+                $self->logger->error(
+                        "Session does not contain _httpSession field. "
+                      . "Portal must be accessed over HTTPS when using CDA with double cookie"
+                );
+                return PE_ERROR;
+            }
         }
 
         my $cdaSession =
