@@ -8,7 +8,7 @@ use Lemonldap::NG::Portal::Main::Constants qw(
   PE_DECRYPTVALUE_SERVICE_NOT_ALLOWED
 );
 
-our $VERSION = '2.0.10';
+our $VERSION = '2.0.12';
 
 extends qw(
   Lemonldap::NG::Portal::Main::Plugin
@@ -39,6 +39,10 @@ sub init {
     );
     return 0 unless $self->rule;
 
+    # Add warning in log
+    $self->logger->warn(
+        "DecryptValue plugin is enabled. You are using a beta version!");
+
     return 1;
 }
 
@@ -86,16 +90,18 @@ sub run {
 
     # Check token
     if ( $self->ottRule->( $req, {} ) ) {
-        my $token = $req->param('token');
-        unless ($token) {
-            $self->userLogger->warn('decryptValue try without token');
-            $msg   = PE_NOTOKEN;
-            $token = $self->ott->createToken();
+        my $token;
+        if ( $token = $req->param('token') ) {
+            unless ( $self->ott->getToken($token) ) {
+                $self->userLogger->warn(
+                    'DecryptValue called with an expired/bad token');
+                $msg   = PE_TOKENEXPIRED;
+                $token = $self->ott->createToken();
+            }
         }
-
-        unless ( $self->ott->getToken($token) ) {
-            $self->userLogger->warn('decryptValue try with expired/bad token');
-            $msg   = PE_TOKENEXPIRED;
+        else {
+            $self->userLogger->warn('DecryptValue called without token');
+            $msg   = PE_NOTOKEN;
             $token = $self->ott->createToken();
         }
 
