@@ -6,32 +6,39 @@ use Mouse;
 use POSIX qw(strftime);
 use Lemonldap::NG::Common::FormEncode;
 use Lemonldap::NG::Portal::Main::Constants qw(
-  PE_BADCREDENTIALS
+  PE_OK
+  PE_MAILOK
+  PE_NOTOKEN
+  PE_MAILERROR
+  PE_PASSWORD_OK
   PE_BADMAILTOKEN
   PE_CAPTCHAEMPTY
   PE_CAPTCHAERROR
-  PE_MAILCONFIRMATION_ALREADY_SENT
-  PE_MAILCONFIRMOK
-  PE_MAILERROR
-  PE_MAILFIRSTACCESS
-  PE_MAILFORMEMPTY
   PE_MAILNOTFOUND
-  PE_MAILOK
-  PE_MALFORMEDUSER
-  PE_NOTOKEN
-  PE_OK
-  PE_PASSWORDFIRSTACCESS
-  PE_PASSWORDFORMEMPTY
-  PE_PASSWORD_MISMATCH
-  PE_PASSWORD_OK
   PE_TOKENEXPIRED
   PE_USERNOTFOUND
+  PE_MAILCONFIRMOK
+  PE_MALFORMEDUSER
+  PE_MAILFORMEMPTY
+  PE_BADCREDENTIALS
+  PE_MAILFIRSTACCESS
+  PE_PASSWORDFORMEMPTY
+  PE_PASSWORD_MISMATCH
+  PE_PASSWORDFIRSTACCESS
+  PE_PP_PASSWORD_TOO_SHORT
+  PE_PP_PASSWORD_TOO_YOUNG
+  PE_PP_PASSWORD_IN_HISTORY
+  PE_MAILCONFIRMATION_ALREADY_SENT
+  PE_PP_INSUFFICIENT_PASSWORD_QUALITY
 );
 
-our $VERSION = '2.0.4';
+our $VERSION = '2.0.12';
 
-extends 'Lemonldap::NG::Portal::Main::Plugin',
-  'Lemonldap::NG::Portal::Lib::SMTP', 'Lemonldap::NG::Portal::Lib::_tokenRule';
+extends qw(
+  Lemonldap::NG::Portal::Lib::SMTP
+  Lemonldap::NG::Portal::Main::Plugin
+  Lemonldap::NG::Portal::Lib::_tokenRule
+);
 
 # PROPERTIES
 
@@ -84,7 +91,7 @@ sub resetPwd {
 
 sub _reset {
     my ( $self, $req ) = @_;
-    my ( $mailToken, %tplPrms );
+    my ($mailToken);
 
     # PASSWORD CHANGE FORM => changePwd()
     if (
@@ -126,7 +133,6 @@ sub _reset {
 
         # Restore pdata if any
         $req->pdata( $mailSession->data->{_pdata} || {} );
-        $mailSession->remove;
         $searchByMail = 0 unless ( $req->{user} =~ /\@/ );
     }
 
@@ -184,14 +190,14 @@ sub _reset {
 
     # Search user in database
     $req->steps( [
-            'getUser',                  'setSessionInfo',
-            'setMacros',                'setGroups',
-            'setPersistentSessionInfo', 'setLocalGroups'
+            'getUser',                 'setSessionInfo',
+            $self->p->groupsAndMacros, 'setPersistentSessionInfo',
+            'setLocalGroups'
         ]
     );
     if ( my $error = $self->p->process( $req, useMail => $searchByMail ) ) {
         if ( $error == PE_USERNOTFOUND or $error == PE_BADCREDENTIALS ) {
-            $self->userLogger->warn( 'Reset asked for an unvalid user ('
+            $self->userLogger->warn( 'Reset asked for an invalid user ('
                   . $req->param('mail')
                   . ')' );
 
@@ -317,7 +323,6 @@ sub _reset {
           );
 
         # Build mail content
-        $tplPrms{MAIN_LOGO} = $self->conf->{portalMainLogo};
         my $tr      = $self->translate($req);
         my $subject = $self->conf->{mailConfirmSubject};
         unless ($subject) {
@@ -330,24 +335,33 @@ sub _reset {
 
             # We use a specific text message, no html
             $body = $self->conf->{mailConfirmBody};
+
+            # Replace variables in body
+            $body =~ s/\$expMailDate/$req->data->{expMailDate}/ge;
+            $body =~ s/\$expMailTime/$req->data->{expMailTime}/ge;
+            $body =~ s/\$url/$url/g;
+            $body =~ s/\$(\w+)/$req->{sessionInfo}->{$1} || ''/ge;
+
         }
         else {
 
             # Use HTML template
-            $body = $self->loadTemplate(
+            $body = $self->loadMailTemplate(
                 $req,
                 'mail_confirm',
                 filter => $tr,
-                params => \%tplPrms
+                params => {
+                    expMailDate => $req->data->{expMailDate},
+                    expMailTime => $req->data->{expMailTime},
+                    url         => $url,
+                },
             );
             $html = 1;
         }
 
-        # Replace variables in body
-        $body =~ s/\$expMailDate/$req->data->{expMailDate}/ge;
-        $body =~ s/\$expMailTime/$req->data->{expMailTime}/ge;
-        $body =~ s/\$url/$url/g;
-        $body =~ s/\$(\w+)/$req->{sessionInfo}->{$1} || ''/ge;
+        $self->logger->info( "User "
+              . $req->data->{mailAddress}
+              . " is trying to reset his/her password" );
 
         # Send mail
         unless (
@@ -405,6 +419,23 @@ sub changePwd {
         return PE_NOTOKEN;
     }
 
+    # Remove the mail token session if mail token is provided
+    my $mailToken = $req->param('mail_token');
+    if ($mailToken) {
+        $self->logger->debug("Token given for password reset: $mailToken");
+
+        # Check if token is valid
+        my $mailSession =
+          $self->p->getApacheSession( $mailToken, kind => "TOKEN" );
+        unless ($mailSession) {
+            $self->userLogger->warn('Bad reset token');
+            return PE_BADMAILTOKEN;
+        }
+
+        $self->logger->debug("Delete token $mailToken");
+        $mailSession->remove;
+    }
+
     # Check if user wants to generate the new password
     if ( $req->param('reset') ) {
         $self->logger->debug(
@@ -436,19 +467,32 @@ sub changePwd {
         }
     }
 
-    # Modify the password TODO: change this
-    # Populate $req->{user} for logging purpose
-    my $tmp = $self->conf->{portalRequireOldPassword};
-    $self->conf->{portalRequireOldPassword} = 0;
+    # Check password quality
+    require Lemonldap::NG::Portal::Password::Base;
+    my $cpq =
+      $self->Lemonldap::NG::Portal::Password::Base::checkPasswordQuality(
+        $req->data->{newpassword} );
+    unless ( $cpq == PE_OK ) {
+        $self->ott->setToken( $req, $req->sessionInfo );
+        return $cpq;
+    }
+
     $req->user( $req->{sessionInfo}->{_user} );
     my $result =
-      $self->p->_passwordDB->modifyPassword( $req,
+      $self->p->_passwordDB->setNewPassword( $req,
         $req->data->{newpassword}, 1 );
     $req->{user} = undef;
-    $self->conf->{portalRequireOldPassword} = $tmp;
 
     # Mail token can be used only one time, delete the session if all is ok
-    return $result unless ( $result == PE_PASSWORD_OK or $result == PE_OK );
+    unless ( $result == PE_PASSWORD_OK or $result == PE_OK ) {
+        $self->ott->setToken( $req, $req->sessionInfo );
+        return $result;
+    }
+
+    my $userlog = $req->sessionInfo->{ $self->conf->{whatToTrace} };
+    my $iplog   = $req->sessionInfo->{ipAddr};
+    $self->userLogger->notice("Password changed for $userlog ($iplog)")
+      if ( defined $userlog and $iplog );
 
     # Send mail containing the new password
     $req->data->{mailAddress} ||=
@@ -456,7 +500,6 @@ sub changePwd {
         $req->{sessionInfo}->{ $self->conf->{mailSessionKey} } );
 
     # Build mail content
-    $tplPrms{MAIN_LOGO} = $self->conf->{portalMainLogo};
     my $tr      = $self->translate($req);
     my $subject = $self->conf->{mailSubject};
     unless ($subject) {
@@ -465,28 +508,32 @@ sub changePwd {
     }
     my $body;
     my $html;
+    my $password = $req->data->{newpassword};
+
     if ( $self->conf->{mailBody} ) {
 
         # We use a specific text message, no html
         $body = $self->conf->{mailBody};
+
+        # Replace variables in body
+        $body =~ s/\$password/$password/g;
+        $body =~ s/\$(\w+)/$req->{sessionInfo}->{$1} || ''/ge;
+
     }
     else {
 
         # Use HTML template
-        $body = $self->loadTemplate(
+        $body = $self->loadMailTemplate(
             $req,
             'mail_password',
             filter => $tr,
-            params => \%tplPrms
+            params => {
+                %tplPrms, password => $password,
+            },
         );
         $html = 1;
     }
 
-    # Replace variables in body
-    my $password = $req->data->{newpassword};
-    $body =~ s/\$password/$password/g;
-    $body =~ s/\$(\w+)/$req->{sessionInfo}->{$1} || ''/ge;
-
     # Send mail
     return PE_MAILERROR
       unless $self->send_mail( $req->data->{mailAddress}, $subject, $body,
@@ -508,7 +555,11 @@ sub setSecurity {
 
 sub display {
     my ( $self, $req ) = @_;
+    my $speChars = $self->conf->{passwordPolicySpecialChar};
+    $speChars =~ s/\s+/ /g;
+    $speChars =~ s/(?:^\s|\s$)//g;
     $self->logger->debug( 'Display called with code: ' . $req->error );
+
     my %tplPrm = (
         SKIN_PATH       => $self->conf->{staticPrefix},
         SKIN            => $self->p->getSkin($req),
@@ -524,8 +575,7 @@ sub display {
         STARTMAILTIME   => $req->data->{startMailTime},
         MAILALREADYSENT => $req->data->{mailAlreadySent},
         MAIL            => (
-            $self->p->checkXSSAttack( 'mail', $req->{user} )
-            ? ''
+              $self->p->checkXSSAttack( 'mail', $req->{user} ) ? ''
             : $req->{user}
         ),
         DISPLAY_FORM            => 0,
@@ -533,6 +583,19 @@ sub display {
         DISPLAY_CONFIRMMAILSENT => 0,
         DISPLAY_MAILSENT        => 0,
         DISPLAY_PASSWORD_FORM   => 0,
+        DISPLAY_PPOLICY         => $self->conf->{portalDisplayPasswordPolicy},
+        PPOLICY_MINSIZE         => $self->conf->{passwordPolicyMinSize},
+        PPOLICY_MINLOWER        => $self->conf->{passwordPolicyMinLower},
+        PPOLICY_MINUPPER        => $self->conf->{passwordPolicyMinUpper},
+        PPOLICY_MINDIGIT        => $self->conf->{passwordPolicyMinDigit},
+        PPOLICY_ALLOWEDSPECHAR  => $speChars,
+        (
+            $speChars
+            ? ( PPOLICY_MINSPECHAR => $self->conf->{passwordPolicyMinSpeChar} )
+            : ()
+        ),
+        DISPLAY_GENERATE_PASSWORD =>
+          $self->conf->{portalDisplayGeneratePassword},
     );
     if ( $req->data->{mailToken}
         and
@@ -593,9 +656,15 @@ sub display {
         $tplPrm{DISPLAY_PASSWORD_FORM} = 1;
     }
 
-    # Display password change form again if passwords mismatch
+    # Display password change form again
+    # - if passwords mismatch
+    # - if password quality check fail
     elsif ($req->error == PE_PASSWORDFORMEMPTY
-        || $req->error == PE_PASSWORD_MISMATCH )
+        || $req->error == PE_PASSWORD_MISMATCH
+        || $req->error == PE_PP_INSUFFICIENT_PASSWORD_QUALITY
+        || $req->error == PE_PP_PASSWORD_TOO_SHORT
+        || $req->error == PE_PP_PASSWORD_TOO_YOUNG
+        || $req->error == PE_PP_PASSWORD_IN_HISTORY )
     {
         $self->logger->debug('Display password form');
         $tplPrm{DISPLAY_PASSWORD_FORM} = $req->sessionInfo->{pwdAllowed};
