@@ -7,7 +7,7 @@ use Lemonldap::NG::Portal::Main::Constants qw(PE_OK);
 
 extends 'Lemonldap::NG::Portal::Lib::LDAP';
 
-our $VERSION = '2.0.6';
+our $VERSION = '2.0.12';
 
 has ldapGroupAttributeNameSearch => (
     is      => 'rw',
@@ -39,11 +39,19 @@ sub setSessionInfo {
     my %vars = ( %{ $self->conf->{exportedVars} },
         %{ $self->conf->{ldapExportedVars} } );
     while ( my ( $k, $v ) = each %vars ) {
-        $req->sessionInfo->{$k} =
-          $self->ldap->getLdapValue( $req->data->{ldapentry}, $v ) || "";
+
+        my $value = $self->ldap->getLdapValue( $req->data->{ldapentry}, $v );
+
+        # getLdapValue returns an empty string for missing attribute
+        # but we really want to return undef so they don't get stored in session
+        # This has to be a string comparison because "0" is a valid attribute
+        # value. See #2403
+        $value = undef if ( $value eq "" );
+
+        $req->sessionInfo->{$k} = $value;
     }
 
-    PE_OK;
+    return PE_OK;
 }
 
 # Load all groups in $groups.
@@ -93,7 +101,8 @@ sub setGroups {
 
     $req->{sessionInfo}->{groups}  = $groups;
     $req->{sessionInfo}->{hGroups} = $hGroups;
-    PE_OK;
+
+    return PE_OK;
 }
 
 1;
